#!/bin/bash
LANG=en_US.utf8 
unameout="$(uname -s)"

country=$1
CNTRYISO=$2
ny=$3
ns=$4
ist=$5
clmyrstrt=$6
clmyrend=$7
dr1=$8
dr2=$9
stnmarkopt=${10}

if [ $unameout = "Darwin" ]; then
    opt="bk"
fi

if [ "$#" -ne 10 ]; then
    echo 
    echo "***************************************************************************************************************************"
    echo "----------------- You need to provide 9 arguments -------------------------------------------------------------------------"
    echo "***************************************************************************************************************************"
    echo 
    
    echo "1) country           = name of the country to monitor"
    echo "2) CNTRYISO          = country international ISO CODE eg : ETH (for ethiopia) or NGA(for nigeria), ... "
    echo "3) nyr               = total number of year in records"
    echo "4) nstations         = total number of stations in the records"
    echo "5) yrdtastrt         = first year (YYYY) of in the records" 
    echo "6) climyrstrt        = first year (YYYY) of the period to consider as climatology"
    echo "7) climyrend         = last year (YYYY) of the period to consider as climatology"
    echo "8) season_begin_date = start date (YYYYMMDD) of the period to monitor"
    echo "9) season_end_date   = end date (YYYYMMDD) of the period to monitor"
    echo 
    echo "The action should be like ..."
    echo
    echo "./do_all.sh ethiopia ETH 50 36 1968 1983 2012 20180101 20180131"
    echo
    exit
fi


dr=${dr1}"-"${dr2}
echo "Number of Years = " $ny
echo "Number of Stns = " $ns
echo "Start Year Data = " $ist
echo "Climatological Period  from $clmyrstrt to $clmyrend" 
echo "Data Period = " $dr
echo $dr > date_range
nd=365
td=$(($nd * $ny))
#
echo
echo
echo 'Reading date range ...'
echo
echo
sleep 1
bd=`cut -c1-8   date_range`
ed=`cut -c10-17 date_range`
# convert from YYYYMMDD to YYYYMONDD format
if [ $unameout = "Darwin" ]; then
    opt="bk"  
  bdate=`date -j -f "%Y%m%d" $bd +%d%b%Y`
  edate=`date -j -f "%Y%m%d" $ed +%d%b%Y`
  icur=`date -j -f "%Y%m%d" $bd +%Y`
  strtday=$(( ( $(date +%s) - $(date -j -f "%Y%m%d" $dr1 +%s) ) /(24 * 60 * 60 ) ))
  endday=$(( ( $(date +%s) - $(date -j -f "%Y%m%d" $dr2 +%s) ) /(24 * 60 * 60 ) ))
else
  bdate=`LC_ALL=en_US.utf8 date --date="$bd" +"%d%b%Y"`
  edate=`LC_ALL=en_US.utf8 date --date="$ed" +"%d%b%Y"`
  icur=`LC_ALL=en_US.utf8 date --date="$bd" +"%Y"`
  strtday=$(( ( $(date +%s) - $(date -d "$dr1" +%s) ) /(24 * 60 * 60 ) ))
  endday=$(( ( $(date +%s) - $(date -d "$dr2" +%s) ) /(24 * 60 * 60 ) ))
fi
# compute the number of days in the date range
  echo $bdate --- $edate
#   exit
#   echo ++ $strtday *--* $endday ++
#   exit
  nd=$(($strtday - $endday))
  ndays=`expr $nd + 1`
  nd=$ndays
  echo $ndays "days in the period"
sleep 1

# echo $ny $ns $ist $td
#
cd images
rm -f *.png
cd ../work

echo
echo ' Creating controlling files to control the Climatological data ...'
echo
sleep 1

######### control file to manage the Map plotting of the station's clim daily data #############
cp ../src/template_data_clim_for_map.ctl data_clim_for_map.ctl
sed -i $opt "s/ISTR/$icur/g" data_clim_for_map.ctl
rm -f *.ctlbk

stnmap -i data_clim_for_map.ctl

######### control file to manage the time series plotting of the station's clim daily data #############
outfile="short_stn_name.txt"
cp ../src/template_daily_clim.ctl daily_clim.ctl
sed -i $opt "s/ICURR/$icur/g" daily_clim.ctl
sed -i $opt "s/NSTNS/$ns/g"   daily_clim.ctl
rm -f *.ctlbk

cat daily_clim.ctl $outfile > temp.ctl
mv temp.ctl daily_clim.ctl
echo "ENDVARS" >> daily_clim.ctl
#
#

#++++PLOTTING PART+++++++++++++++++++++++++++++++++++++++++
echo
echo
echo ' Creating Plots ...'
echo
echo
sleep 1

if [ $unameout = "Darwin" ]; then
    strtday=$(( ( $(date +%s) - $(date -j -f "%Y%m%d" $dr1 +%s) ) /(24 * 60 * 60 ) ))
    endday=$(( ( $(date +%s) - $(date -j -f "%Y%m%d" $dr2 +%s) ) /(24 * 60 * 60 ) ))
else
    strtday=$(( ( $(date +%s) - $(date -d "$dr1" +%s) ) /(24 * 60 * 60 ) ))
    endday=$(( ( $(date +%s) - $(date -d "$dr2" +%s) ) /(24 * 60 * 60 ) ))
fi

################## GENERATES DISTRIBUTIONS MAPS OF RAIN AMOUNT ##################################
echo "--------------------- ${endday} ${strtday} --------------------"


echo $ndays > numdays
if [ $unameout = "Darwin" ]; then
    dy1=`date -v -${strtday}d +%d%b%Y`
    dy2=`date -v -${endday}d +%d%b%Y`
    
    dy1clm=`date -v -${strtday}d +%d%b`
    dy2clm=`date -v -${endday}d +%d%b`
else
    dy1=`LC_ALL=en_US.utf8 date --date "${strtday} day ago" "+%d%b%Y"`
    dy2=`LC_ALL=en_US.utf8 date --date "${endday} day ago" "+%d%b%Y"`

    dy1clm=`LC_ALL=en_US.utf8 date --date "${strtday} day ago" "+%d%b"`
    dy2clm=`LC_ALL=en_US.utf8 date --date "${endday} day ago" "+%d%b"`
fi
echo $dy1 $dy2 $dy1clm $dy2clm $ndays

cp ../src/colors .
cp ../src/xcbar_cpt.gs .
cp ../src/template_plot_stn_rev.gs plots.gs


sed -i $opt 's/xday/'${ndays}'/g' plots.gs
sed -i $opt 's/date1/'${dy1}'/g' plots.gs
sed -i $opt 's/date2/'${dy2}'/g' plots.gs
sed -i $opt 's/dateclm1/'${dy1clm}'/g' plots.gs
sed -i $opt 's/dateclm2/'${dy2clm}'/g' plots.gs
sed -i $opt 's/tpr/'$ndays'/g' plots.gs
sed -i $opt 's/CONTRY/'${country}'/g' plots.gs
sed -i $opt 's/CNTRY/'$CNTRYISO'/g' plots.gs
sed -i $opt 's/STNMRKOPT/'$stnmarkopt'/g' plots.gs
grads -bpc plots.gs
rm -f *.gsbk

mv ${country}_rev_stn_*.png                     ../images/.
cp ../images/${country}_rev_stn_*.png           ../ToolKit/spatial_maps/${country}/.


################## GENERATES STATION TIME SERIES AND COMPUTE SOME RAIN INDEXES  ##################################
nrc=0
stid=1
rm -f stn_rain_index.txt 
while read lat lon lname sname 
    do

    nrc=$(($nrc + 1))
    if [ $nrc -gt 1 ] 
        then
        stidr=$(($stid - 1))
        echo
        echo
        echo " now processing"  $sname
        sleep 1
        echo
        echo
        cp ../src/template_stn_rain_index.gs calc_rain_index.gs
        sed -i $opt "s/STNAME/$sname/g" calc_rain_index.gs
        sed -i $opt "s/STID/$stidr/g" calc_rain_index.gs
        sed -i $opt "s/SLAT/$lat/g" calc_rain_index.gs
        sed -i $opt "s/SLON/$lon/g" calc_rain_index.gs
        sed -i $opt "s/date1/$bdate/g" calc_rain_index.gs
        sed -i $opt "s/date2/$edate/g" calc_rain_index.gs
        sed -i $opt "s/CLMYEAR1/$clmyrstrt/g" calc_rain_index.gs
        sed -i $opt "s/CLMYEAR2/$clmyrend/g" calc_rain_index.gs
        sed -i $opt 's/DDMM1/'${dy1clm}'/g' calc_rain_index.gs
        sed -i $opt 's/DDMM2/'${dy2clm}'/g' calc_rain_index.gs
        rm -f *.gsbk
        grads -blc calc_rain_index.gs
        
        cp ../src/template_data_for_plot.gs data_for_plot.gs
        sed -i $opt "s/STNAME/$sname/g" data_for_plot.gs
        sed -i $opt "s/BD/$bdate/g" data_for_plot.gs
        sed -i $opt "s/ED/$edate/g" data_for_plot.gs
        rm -f *.gsbk
        grads -blc data_for_plot.gs
        #
        cp ../src/template_scratch.ctl scratch.ctl
        sed -i $opt "s/NUMDAYS/$ndays/g" scratch.ctl
        sed -i $opt "s/BDATE/$bdate/g" scratch.ctl
        cp ../src/template_scratch.f scratch.f
        sed -i $opt "s/NUMDAYS/$ndays/g" scratch.f
        rm -f *.gsbk *.fbk
        gfortran -o scratch.exe scratch.f
        ./scratch.exe

        cp ../src/template_plot_english.gs plot.gs
        sed -i $opt "s/STNM/$sname/g" plot.gs
        sed -i $opt "s/SLAT/$lat/g" plot.gs
        sed -i $opt "s/SLON/$lon/g" plot.gs
        sed -i $opt "s/YYYY/$icur/g" plot.gs
        sed -i $opt "s/CLMYEAR1/$clmyrstrt/g" plot.gs
        sed -i $opt "s/CLMYEAR2/$clmyrend/g" plot.gs
        rm -f *.gsbk
        grads -blc plot.gs
        
        convert -trim -density 500 ../images/$sname.png ../images/$sname.png

        cp ../images/$sname.png ../ToolKit/ts_maps/${country}/.
        
    fi
    stid=$(($stid + 1))
done < stns_latlon.txt


################## GENERATES THE DISTRIBUTIONS MAPS OF THE ABOVE COMPUTED RAIN INDEXES  ##################################
cp ../src/convert_stn_rain_index_bin.sh .
bash convert_stn_rain_index_bin.sh 

cp  ../src/template_plot_stn_rev_rain_index.gs plots_rain_index.gs
sed -i $opt 's/xday/'${ndays}'/g' plots_rain_index.gs
sed -i $opt 's/tpr/'$ndays'/g' plots_rain_index.gs
sed -i $opt 's/date1/'${dy1}'/g' plots_rain_index.gs
sed -i $opt 's/date2/'${dy2}'/g' plots_rain_index.gs
sed -i $opt 's/CONTRY/'$country'/g' plots_rain_index.gs
sed -i $opt 's/CNTRY/'$CNTRYISO'/g' plots_rain_index.gs
sed -i $opt 's/STNMRKOPT/'$stnmarkopt'/g' plots_rain_index.gs
rm -f *.gsbk
grads -bpc plots_rain_index.gs

mv ${country}_rev_stn_*.png                     ../images/.
cp ../images/${country}_rev_stn_*.png           ../ToolKit/spatial_maps/${country}/.

#
rm  max_file ../date_range
rm data_clim_for_map.ctl daily_clim.ctl *.gs *.gsbk

echo
echo
echo ' Finished Processing All the Plots ... BYE!'
echo
sleep 2
