#!/bin/bash
LANG=en_US.utf8 
unameout="$(uname -s)"

contry=$1     # country name
CNTRYISO=$2   # country international ISO CODE

country_geo_coord_file="input_data/${contry}_latlon"
#readarray -t ctryinfo < "${country_geo_coord_file}"
i=0;while IFS= read -r ctryinfo[i++]; do :; done <${country_geo_coord_file}

echo ${ctryinfo[0]} | awk '{for (f=1;f<=NF;f++) col[f] = col[f]":"$f} END {for (f=1;f<=NF;f++) print col[f]}' | tr ':' ' ' > bd.txt
declare -a valn0=(`cat bd.txt`)
rm -f bd.txt
    

rawminlat=${valn0[1]} # S_lat
rawmaxlat=${valn0[2]} # N_lat
rawminlon=${valn0[3]} # W_lon
rawmaxlon=${valn0[4]} # E_lon


echo ""

cp ToolKit/src/save_data_in_grads.gs .
cp ToolKit/src/parsestr.gsf .
cp ToolKit/src/qdims.gsf .
shpfile="ToolKit/gis_resources/countries/${CNTRYISO}_adm/${CNTRYISO}_adm0.shp"         # shapefile of the region of interest


for rsl in 0p036 0p5
    do
    echo "### The extend to be used to generate country mask to fit the dataset with a spatial resolution of ${rsl}, are : "
    if [ $rsl = "0p036" ]  ; then dtarsl=0.036 ; dtaminlon=-179.982 ; dtaminlat=-55.314 ; fi
    if [ $rsl = "0p5" ]    ; then dtarsl=0.5   ; dtaminlon=0.25     ; dtaminlat=-89.75  ; fi
    prefix_outfile="${contry}_${rsl}"

    nptsrawL=`bc <<< "(($rawminlon - ($dtaminlon))/$dtarsl)" `
    nptsrawR=`bc <<< "(($rawmaxlon - ($dtaminlon))/$dtarsl)" `
    
    newminlon=`bc -l <<< "(($nptsrawL -1 )*$dtarsl)+($dtaminlon)" `
    newmaxlon=`bc -l <<< "(($nptsrawR + 1 )*$dtarsl)+($dtaminlon)" `

    
    
    nptsrawS=`bc <<< "(($rawminlat - ($dtaminlat))/$dtarsl)" `
    nptsrawN=`bc <<< "(($rawmaxlat - ($dtaminlat))/$dtarsl)" `
    
    newminlat=`bc -l <<< "(($nptsrawS -1 )*$dtarsl)+($dtaminlat)" `
    newmaxlat=`bc -l <<< "(($nptsrawN + 1 )*$dtarsl)+($dtaminlat)" `
    

    echo "================== Western Longitude  ================" $newminlon 
    echo "================== Eastern Longitude  ================" $newmaxlon
    echo "================== Southern Latitude  ================" $newminlat 
    echo "================== Northern Latitude ================" $newmaxlat
    echo ""


    lonW=$newminlon                              #
    lonE=$newmaxlon                              # Extend of the domain surronding the target area, 
    latS=$newminlat                              # these values must be expressed in georeferenced units.
    latN=$newmaxlat                              #

    #####################################################################################################################
    # Do not change anything here, the following actions are important for the sharpness of the mask you want to create.#
    #####################################################################################################################
    shift_set=`bc -l <<< "$dtarsl/2.0"`                                                                                #
    lonW0=`bc -l <<< "$lonW - $shift_set"`                                                                              #
    lonE0=`bc -l <<< "$lonE + $shift_set"`                                                                              #
    latS0=`bc -l <<< "$latS - $shift_set"`                                                                              #
    latN0=`bc -l <<< "$latN + $shift_set"`                                                                              #
    #####################################################################################################################

    # ############################## Using GDAL commands lines to generated two different mask ##########################
    #  1) Generating a fist mask by burning the fixed value, here 1, inside the provided polygon.
    gdal_rasterize -burn 1 -te $lonW0 $latS0 $lonE0 $latN0 -tr $dtarsl $dtarsl -of netCDF $shpfile data_mask_1.nc 

    #  2) Generating a second mask by burning the value -1 all parts of the domain but not inside the provided polygon.
    gdal_rasterize -i -burn -1 -te $lonW0 $latS0 $lonE0 $latN0 -tr $dtarsl $dtarsl -of netCDF $shpfile data_mask_2.nc 
    #--------------------------------------------------------------------------------------------------------------------
    
    ######################### Merging the two .nc files  to generate a single .ctl file #################################
cat>select_area_rain.gs<<eofGS
'reinit'
'set display color white'
'c'
*---------- ------------
'sdfopen data_mask_1.nc'
'sdfopen data_mask_2.nc'
'define mask1=band1.1'
'define mask2=band1.2'

'define mask1 = const(mask1,0,-u)'
'define mask2 = const(mask2,0,-u)'

'define mask = mask1 + mask2'

'set lat $latS $latN'
'set lon $lonW $lonE'

'save_data_in_grads -v mask -n "mask" -d "background mask data" -u -9999.0 -o $prefix_outfile'
'quit'
eofGS

grads -blc select_area_rain.gs
rm -f select_area_rain.gs data_mask_1.nc data_mask_2.nc
#--------------------------------------------------------------------------------------------------------------------
if [ $unameout = "Darwin" ]; then
    opt="bk"
fi

#    sed -i $opt "/title /c\title Mask data" $prefix_outfile.ctl
    sed -i $opt "s/.*title.*/title Mask data./" $prefix_outfile.ctl
    rm -f *.ctlbk

rm -f ToolKit/gis_resources/country_masks${rsl}/$prefix_outfile.ctl
rm -f ToolKit/gis_resources/country_masks${rsl}/$prefix_outfile.ctlbk
rm -f ToolKit/gis_resources/country_masks${rsl}/$prefix_outfile.dat

    if [ ! -f "ToolKit/gis_resources/country_masks${rsl}/$prefix_outfile.ctl" ] && [ ! -f "ToolKit/gis_resources/country_masks${rsl}/$prefix_outfile.dat" ] ; then 
        cp -rf $prefix_outfile*  ToolKit/gis_resources/country_masks${rsl}/.
    else
        echo "The target file is already on the local distant"
        ls -lrt ToolKit/gis_resources/country_masks${rsl}/$prefix_outfile.ctl
        ls -lrt ToolKit/gis_resources/country_masks${rsl}/$prefix_outfile.dat
    fi

    rm -f $prefix_outfile.ctl $prefix_outfile.dat
    rm -f *.ctlbk

done

rm -f save_data_in_grads.gs parsestr.gsf qdims.gsf
