#!/bin/bash

#######################################################################################################
filetxt="stn_rain_index.txt"
prefix="stn_rain_index"
ymissval=-99999.00

####################################################################################
# -----  rewrite the data again (with format)
nstations=$(cat "${filetxt}" | wc -l)  	     #### Total Number of Row in the File
nout=$nstations 			                 # FINAL NUMBER OF ROWS of the filetxt
nrec=$[$nout/$nstations]
# echo $nout


cat > rewrite.f90 << eoffortran

program rewritedata

	integer, parameter :: ir=kkkk
	integer (kind=4) :: year (1:ir), mon(1:ir), day(1:ir)
	character (len=17) :: stn(1:ir)
	real (kind=4) :: lat(1:ir), lon(1:ir)
	real (kind=4) :: var(1:ir,1:8)
	integer (kind=4) :: time(1:ir), nlev, nflag

	 open(10,file='${filetxt}')
	
	 do i=1,ir
	 read(10,*) year(i), mon(i), day(i), stn(i), lat(i), lon(i), var(i,:)
	 enddo 
	 
	! putting format to the data

        open(20,file='good_format_newfile_ready_to_bin.txt')
 	
	do i=1,ir
	 write(20,200) year(i), mon(i), day(i), stn(i), lat(i), lon(i), var(i,:)
	enddo
	
200     format(i4,2x,i3,2x,i3,4x,a4,6x,f6.2,5x,f6.2,3x,8f9.3) 

end program

eoffortran

line1=$(grep ir=kkkk rewrite.f90)
line1a="        integer, parameter :: ir= ${nout} "
sed "s/$line1/$line1a/" rewrite.f90 > rewritedata.f90

gfortran rewritedata.f90
./a.out 

rm -f a.out rewritedata.f90 rewrite.f90
# exit
####################################################################################


cat > mybin.c << EOF
	#include <stdio.h>

	struct rpthdr {
	  char id[8];           /* Character station id  */
	  float lat;            /* Latitude of report             */
	  float lon;            /* Longitude of report            */
	  float t;              /* Time in relative grid units    */
	  int  nlev;            /* Number of levels following     */
	  int flag;             /* Level independent var set flag */
	} hdr;

	void main () {

	  FILE *ifile, *ofile;

	  char rec[150];
	  int flag,year,month,day,yrsav,mnsav,dysav,i,idx;
	  float val1,val2,val3,val4,val5,val6,val7,val8;

	  ifile = fopen ("good_format_newfile_ready_to_bin.txt","r");
	  ofile = fopen ("output_good_format.bin","wb");

	  if (ifile==NULL || ofile==NULL) {
	    printf ("Error input file\n");
	    return;
	  }

	  flag = 1;

	  while (fgets(rec,149,ifile) != NULL){

	    sscanf (rec,"%i %i %i",&year,&month,&day);
	    sscanf (rec+28," %g %g %g %g %g %g %g %g %g %g",&hdr.lat,&hdr.lon,&val1,&val2,&val3,&val4,&val5,&val6,&val7,&val8);

	    for (i=0;i<8;i++) 
	     hdr.id[i] = rec[i+21];

	     if(flag) {
	     yrsav = year;
	     mnsav = month;
	     dysav = day;
	     flag = 0;
	     }

	     if (yrsav!=year || mnsav!=month || dysav!=day) {
	     hdr.nlev = 0;
	     fwrite (&hdr,sizeof(struct rpthdr), 1, ofile);
	     }
	     yrsav = year; 
	     mnsav = month;
	     dysav = day;

	     hdr.nlev = 1;
	     hdr.flag = 1;
	     hdr.t = 0.0;

	    fwrite (&hdr,sizeof(struct rpthdr), 1, ofile);
        fwrite (&val1,sizeof(float),1, ofile);
        fwrite (&val2,sizeof(float),1, ofile);
        fwrite (&val3,sizeof(float),1, ofile);
        fwrite (&val4,sizeof(float),1, ofile);
        fwrite (&val5,sizeof(float),1, ofile);
        fwrite (&val6,sizeof(float),1, ofile);
        fwrite (&val7,sizeof(float),1, ofile);
        fwrite (&val8,sizeof(float),1, ofile);
         
	  }

	  hdr.nlev = 0;
	  fwrite (&hdr,sizeof(struct rpthdr), 1, ofile);
	}

EOF

##################################################################################################################################################
##################################################################################################################################################
##################################################################################################################################################

gfortran mybin.c
./a.out
rm -f a.out mybin.c
# exit
mv output_good_format.bin ${prefix}.bin
# exit
rm -f newfile_ready_to_bin.txt good_format_newfile_ready_to_bin.txt

# exit
#################################################################################
#################################################################################


cat > ${prefix}.ctl << EOF
DSET ${prefix}.bin
DTYPE station
stnmap ${prefix}.map
UNDEF  $ymissval 
TITLE station Data
TDEF $nrec LINEAR 01Jan1981 1dy
VARS 8
totnrd 0 99     Total Number of Rainy Days (R>1mm)
clmtotnrd 0 99  Clim. Total Number of Rainy Days
tothrd 0 99     Total Number of Heavy Rain Days (R>10mm)
totvhrd 0 99    Total Number of Very Heavy Rain Days (R>20mm)
maxcwd 0 99     Max Number of Consec. Wet Days
maxcdd 0 99     Max Number of Consec. Dry Days
maxchrd 0 99    Max Number of Consec. Heavy Rain Days
maxcvhrd 0 99   Max Number of Consec. Very Heavy Rain Days  
ENDVARS
EOF

stnmap -i ${prefix}.ctl

#################################################################################
#################################################################################

