#!/bin/sh
#
ny=$1
ns=$2
ist=$3
dr1=$4
dr2=$5
dr=${dr1}"-"${dr2}
echo "Number of Years = " $ny
echo "Number of Stns = " $ns
echo "Start Year Data = " $ist
echo "Data Period = " $dr
echo $dr > date_range
nd=365
td=$(($nd * $ny))
#
echo
echo
echo 'Reading date range ...'
echo
echo
sleep 2
bd=`cut -c1-8   date_range`
ed=`cut -c10-17 date_range`
# convert from YYYYMMDD to YYYYMONDD format
bdate=`LC_ALL=en_US.utf8 date --date="$bd" +"%d%b%Y"`
edate=`LC_ALL=en_US.utf8 date --date="$ed" +"%d%b%Y"`
icur=`LC_ALL=en_US.utf8 date --date="$bd" +"%Y"`
# compute the number of days in the date range
d1=`cut -c3-8   date_range`
d2=`cut -c12-17 date_range`
echo $d1 $d2
nd=$(( ($(LC_ALL=de_DE.utf8 date --date="$d2" +%s) - $(LC_ALL=de_DE.utf8 date --date="$d1" +%s) )/(60*60*24) ))
ndays=`expr $nd + 1`
echo $ndays " days in the period"
sleep 2
#
echo $ny $ns $ist $td
#
echo
echo
echo 'Reformatting Data ...'
echo
echo
sleep 3
#
cd work
rm reformat.f
cp ../src/template_reformat.f reformat.f
cp ../input_data/stations.txt .
sed -i "s/NYRS/$ny/g" reformat.f
sed -i "s/NSTNS/$ns/g" reformat.f
sed -i "s/STYR/$ist/g" reformat.f
gfortran -o reformat.exe reformat.f
./reformat.exe
#
sleep 2
echo
echo
echo 'Computing Climatology ...'
echo
echo 'Writing Data in binary and ASCII formats  ...'
echo
sleep 2
#
cp ../src/template_daily_clim.f daily_clim.f
sed -i "s/NYRS/$ny/g" daily_clim.f
sed -i "s/NSTNS/$ns/g" daily_clim.f
sed -i "s/ISTR/$ist/g" daily_clim.f
gfortran -o daily_clim.exe daily_clim.f
./daily_clim.exe
mv *.bin ../output_data/
mv *.asc ../output_data/
#
echo
echo
echo ' Done Writing Data ...'
echo
echo ' Creating controlling files ...'
echo
echo
sleep 4
#
outfile="short_stn_name.txt"
cp ../input_data/stns_latlon.txt .
awk -v val=$outfile '{print $4 "  0 99 ch01  p" > val}' stns_latlon.txt
sed -i -e "1d" $outfile
ns=`wc $outfile | awk '{print $1 }'`
#
cp ../src/template_daily_data.ctl daily_data.ctl
sed -i "s/ISTR/$ist/g" daily_data.ctl
sed -i "s/NDAYS/$td/g" daily_data.ctl
sed -i "s/NSTNS/$ns/g" daily_data.ctl
cat daily_data.ctl $outfile > temp.ctl
mv temp.ctl daily_data.ctl
echo "ENDVARS" >> daily_data.ctl
#
cp ../src/template_daily_clim.ctl daily_clim.ctl
sed -i "s/ICURR/$icur/g" daily_clim.ctl
sed -i "s/NSTNS/$ns/g"   daily_clim.ctl
cat daily_clim.ctl $outfile > temp.ctl
mv temp.ctl daily_clim.ctl
echo "ENDVARS" >> daily_clim.ctl
#
#
#++++PLOTTING PART+++++++++++++++++++++++++++++++++++++++++
echo
echo
echo ' Creating Plots ...'
echo
echo
sleep 4
#
#
nrc=0
while read lat lon lname sname ; do
#
nrc=`expr $nrc + 1`
if [ $nrc -gt 1 ] 
then
echo
echo
echo " now processing"  $sname
sleep 2
echo
echo
cp ../src/template_data_for_plot.gs data_for_plot.gs
sed -i "s/STNAME/$sname/g" data_for_plot.gs
sed -i "s/BD/$bdate/g" data_for_plot.gs
sed -i "s/ED/$edate/g" data_for_plot.gs
grads -blc data_for_plot.gs
#
cp ../src/template_scratch.ctl scratch.ctl
sed -i "s/NUMDAYS/$ndays/g" scratch.ctl
sed -i "s/BDATE/$bdate/g" scratch.ctl
cp ../src/template_scratch.f scratch.f
sed -i "s/NUMDAYS/$ndays/g" scratch.f
gfortran -o scratch.exe scratch.f
./scratch.exe
cp ../src/template_plot_english.gs plot.gs
sed -i "s/STNM/$sname/g" plot.gs
sed -i "s/YYYY/$icur/g" plot.gs
grads -blc plot.gs
fi
done < stns_latlon.txt
#
rm  max_file date_range
# rm *.ctl *.gs *.txt *.dat *.bin *.f *.exe
echo
echo
echo ' Finished Processing All the Plots ... BYE!'
echo
sleep 3

