#!/bin/bash
LANG=en_US.utf8 
unameout="$(uname -s)"

country=$1
CNTRYISO=$2
nyr=$3
yrdtastrt=$4
climyrstrt=$5
climyrend=$6
prd_bg_date=$7
prd_end_date=$8
stnmarkorshdopt=$9   #1 (map as mark at station location) or 0 (map of interpolatad station data)

    if [ "$#" -lt 8 ] || [ "$#" -gt 9 ]; then
        echo 
        echo "***************************************************************************************************************************"
        echo "----------------- You need to provide 7 arguments -------------------------------------------------------------------------"
        echo "***************************************************************************************************************************"
        echo 
        
        echo "1) country                    = name of the country to monitor"
        echo "2) CNTRYISO                   = country international ISO CODE eg : ETH (for ethiopia) or NGA(for nigeria), ... "
        echo "3) nyr                        = total number of year in records"
        echo "4) yrdtastrt                  = first year (YYYY) of in the records" 
        echo "5) climyrstrt                 = first year (YYYY) of the period to consider as climatology"
        echo "6) climyrend                  = last year (YYYY) of the period to consider as climatology"
        echo "7) prd_bg_date                = start date (YYYYMMDD) of the period to monitor"
        echo "8) prd_end_date               = end date (YYYYMMDD) of the period to monitor"
        echo "9) stnmarkorshdopt (optional) = 1 (map as mark at station location) or 0 (map of interpolatad station data). Default value is 1"
        echo 
        echo "The action should be like ..."
        echo
        echo "./zstart.sh ethiopia ETH 50 1968 1983 2012 20180301 20180531 1"
        echo
        exit
    fi

    if [ "$#" -lt 9 ]; then
        stnmarkorshdopt=1
    fi

cd input_data
cp ${country}_stns_data_cmt.txt stations.txt
cp ${country}_stns_latlon.txt stns_latlon.txt
cd ..

if [ ! -f "ToolKit/ts_maps/${country}/pix_rev_body.html" ]  ; then cp ToolKit/fix/${country}_pix_rev_body.html   ToolKit/ts_maps/${country}/pix_rev_body.html; fi

stn_coord_file="input_data/stns_latlon.txt"
nstations=$(cat "${stn_coord_file}" | wc -l)  	             #### TOTAL NUMBER OF ROW IN THE FILE
nstations=$(($nstations - 1))

################### REFORMATTING THE STATION DATA TO MATCH THE EMBEDDED SCRIPT ########################
rm -Rf work
mkdir work 
cd work
rm -f script_data_prep.sh 
cp ../src/script_data_prep.sh .
bash script_data_prep.sh $nstations $nyr $yrdtastrt $climyrstrt $climyrend
cd ..


#################################### PLOTTING PART######################################################

 bash do_all.sh ${country} ${CNTRYISO} $nyr $nstations $yrdtastrt $climyrstrt $climyrend ${prd_bg_date} ${prd_end_date} ${stnmarkorshdopt}


rm -f input_data/stations.txt input_data/stns_latlon.txt 
rm -Rf work

cd ToolKit


if [ $unameout = "Darwin" ]; then
    opt="bk"
	strtday=$(( ( $(date +%s) - $(date -j -f "%Y%m%d" ${prd_bg_date} +%s) ) /(24 * 60 * 60 ) ))
	endday=$(( ( $(date +%s) - $(date -j -f "%Y%m%d" ${prd_end_date} +%s) ) /(24 * 60 * 60 ) ))
	
    ddmmmyyyy1=`date -v -${strtday}d +%d%b%Y`
    ddmmmyyyy2=`date -v -${endday}d +%d%b%Y`
else
	strtday=$(( ( $(date +%s) - $(date -d "${prd_bg_date}" +%s) ) /(24 * 60 * 60 ) ))
	endday=$(( ( $(date +%s) - $(date -d "${prd_end_date}" +%s) ) /(24 * 60 * 60 ) ))
	
    ddmmmyyyy1=`LC_ALL=en_US.utf8 date --date "$strtday day ago" "+%d%b%Y"`
    ddmmmyyyy2=`LC_ALL=en_US.utf8 date --date "$endday day ago" "+%d%b%Y"`
fi

cp template_country_prd_rev_index.html ${country}_station_rev_index.html
sed -i $opt "s|COUNTRY|${country}|g;s|ddmmmyyyy1|$ddmmmyyyy1|g;s|ddmmmyyyy2|$ddmmmyyyy2|g" ${country}_station_rev_index.html
rm -f *.htmlbk

# ----------------- Open the html to view the results ---------------------------------------------
if [ $unameout = "Darwin" ]; then
   /Applications/Google\ Chrome.app/Contents/MacOS/Google\ Chrome ${country}_station_rev_index.html&
   open -a Safari ${country}_station_rev_index.html&
else
    '/cygdrive/c/Program Files (x86)/Google/Chrome/Application/chrome'&
        sleep 3
    '/cygdrive/c/Program Files (x86)/Google/Chrome/Application/chrome' '${country}_station_rev_index.html'&
    firefox       ${country}_station_rev_index.html&
    google-chrome ${country}_station_rev_index.html&
fi


echo ""
echo ""
echo "-------------------------------------------------------------------------------"
echo "--------- Process of Climate Monitoring is complete successfully --------------"
echo "---------     Check your browser for results            -----------------------"
echo "-------------------------------------------------------------------------------"
echo ""
echo ""
